﻿/*
===============================================================================
 Name        : lcd.c
 Author      : Ichiro Shirasaka
 Version     : Ver.1.0 2012.09.10
 Copyright   : Copyright (C) 2012 Ichiro Shirasaka
 Description : UIEXB TEST MAIN
===============================================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================
*/

#include "lcd.h"
#include "touch.h"

#define CR 0xd		//キャリッジロターン
#define LF 0xa		//ラインフード
#define FF 0x0c		//ホームフィード
#define PRINTLCD	//標準出力をＬＣＤにアサイン

const uint8_t cgdata[56] = {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
							0x20,0x20,0x20,0x20,0x20,0x20,0x20,0x20,
							0x30,0x30,0x30,0x30,0x30,0x30,0x30,0x30,
							0x38,0x38,0x38,0x38,0x38,0x38,0x38,0x38,
							0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,0x3c,
							0x3e,0x3e,0x3e,0x3e,0x3e,0x3e,0x3e,0x3e,
							0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f,0x3f};

extern volatile int time_count;

/* Initialize LCD */
void init_lcd(void)
{
	write_lcd(0, 0x38);		//8bit 2line Normal mod
	write_lcd(0, 0x39);		//8bit 2line Extend mode
	write_lcd(0, 0x14);		//OSC=183Hz Bias=1/5
	write_lcd(0, 0x7f);		//Contrast = 0x2f（下位）
	write_lcd(0, 0x5e);		//Contrast = 0x2f（上位）
	write_lcd(0, 0x6a);		//Follower : 5V
	write_lcd(0, 0x6b);		//Follower :3.3V
	delay_ms(300);			//Wait Execution
	write_lcd(0, 0x38);		//8bit 2line Normal mod
	write_lcd(0, 0x0c);		//8bit 2line Normal mod
	write_lcd(0, 0x01);		//Clear Display
	return;
}

/* CG DATA Load */
void cgwrite_lcd(void)
{
	short int count;

	for(count=0; count < 56; count++){
		write_lcd(0, 0x40 | count);
		write_lcd(1, cgdata[count]);
	}
}

/* write LCD */
void write_lcd(unsigned int rs, unsigned char data)
{
    I2CWriteLength = 3;		//I2C write transfer byte = 3
    I2CReadLength = 0;		//I2C read transfer byte = 0
    I2CMasterBuffer[0] = 0x7c;		//LCD slave address
    I2CMasterBuffer[1] = rs << 6;	// co=0 rs
    I2CMasterBuffer[2] = data;		//data
    I2CEngine();			//Go I2C engine
    return;
}

/* Wait 1ms interrupt */
void delay_count(void)
{
    if(time_count > 0)
    {
	    time_count--;	//count down by 1ms tick interrupt
    }
}

/* Delay ms  */
void delay_ms(int time_ms)
{
    time_count = time_ms;
    while(time_count > 0);	//wait time_count until 0
}

/* 低レベルIO出力関数 */
#ifdef PRINTLCD
int __sys_write(int iFileHandle, char *pcBuffer, int iLength)
{
	static short int ac = 0;

	while(iLength > 0)
	{
		if(*pcBuffer > 0x1f){			//not control character
			write_lcd(0, 0x80 | ac);
			write_lcd(1, *pcBuffer);	//Display 1byte
			ac++;
			if(ac == 0x50){
				ac = 0;
			}
			else{
				if(ac == 16){
					ac = 0x40;
				}
			}
		}
		else{
			if(*pcBuffer == LF){
				if(ac < 16){
					ac = 0x40;
				}
				else{
					ac = 0x00;
				}
			}
			if(*pcBuffer == FF){
				ac = 0;
			}
		}
		pcBuffer++;
		iLength--;
	}
	return 0;
}
#endif
