﻿/*
===============================================================================
 Name        : main.c
 Author      : Ichiro Shirasaka
 Version     : Ver.1.0 2012.09.10
 Copyright   : Copyright (C) 2012 Ichiro Shirasaka
 Description : UIEXB TEST MAIN
===============================================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================
*/

/********************************************************************
*
*  UIEX Board Test Program
*
*********************************************************************/

#ifdef __USE_CMSIS
#include "LPC11xx.h"
#endif

#include <cr_section_macros.h>
#include <NXP/crp.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "lcd.h"
#include "gpio.h"
#include "touch.h"
#include "bz.h"
#include "rtc.h"
#include "acc.h"
#include "uart.h"
#include "pff.h"
#include "diskio.h"
#include "pwm.h"


// Variable to store CRP value in. Will be placed automatically
// by the linker when "Enable Code Read Protect" selected.
// See crp.h header for more information
__CRP const unsigned int CRP_WORD = CRP_NO_CRP ;


#define SW1P -20	//Touch Slider Pad1
#define SW2P 40		//Pad2
#define SW3P 60		//Pad3
#define SW4P 120	//Pad4
#define PAI 3.1416
#define MFXMAX 390	//Magnetic Field Max. X
#define MFXMIN -705	//Magnetic Field　Min. X
#define MFYMAX 620	//Magnetic Field　Max. Y
#define MFYMIN -666	//Magnetic Field　Min. Y
#define MFZMAX 543	//Magnetic Field　Max. Z
#define MFZMIN -520	//Magnetic Field　Min. Z
#define HEADER 44

extern int slider_pos;	//Touch Slider Position

/* Prototype */
void sitoa(int data, char *buf);
void tch_sw(void);
void tch_slider(void);
void rtc_test(void);
void h_ind(void);
void mf_ind(void);
void t_ind(void);
void rtc_set(void);
void dbufup(short int *dbuf, char n);
void dbufdn(short int *dbuf, char n);
void lcd_test(void);
void p_ind(void);
void sd_test(void);
void read_dir(void);
static void put_rc (FRESULT rc);
void read_data(char *fname);
void open_file(char *fname);
void play_m(signed char n);

const char week[28] = "SUN\0MON\0TUE\0WED\0THU\0FRI\0SAT"; //Use RTC

/* Menu DATA */
const char title[9][16] =  {"LCD Display    ",
							"Touch Switch   ",
							"Touch Slider   ",
							"Calendar Clock ",
							"Clinometer     ",
							"Compasrs       ",
							"Thermometer    ",
							"Music Player   ",
							"Mag sensor data"};

/* SD Card control */
FATFS fs;
DIR dir;      		//Directory object
FILINFO fno;  		//File information
volatile uint8_t lbuf[4][128];	//Read Buffer
volatile int rb_in = 0, rb_out = 0;	//Read in,out counter
char dir_bf[16][13];//Directory File name
int dir_size[16];	//Directory File Size

/* Test Program Main routine */
int main(void)
{
	signed char n = 1, sw = 0, on = 0;

	GPIOInit();										//Initialize GPIO
	if ( I2CInit( (uint32_t)I2CMASTER ) == FALSE )	//initialize I2c
	{
 		while ( 1 );								//Fatal error
	}
	SysTick_Config(SystemCoreClock / 1000);		   //SysTick interrupt=1ms
	delay_ms(500);		//Start up Wait
	adinit();			//Initialize AD converter
	touch_ini();		//Initialize Touch sensor
	init_lcd();			//Initialize LCD logic
	write_lcd(0, 0);	//LCD Clear
	timer_ini();		//Set up Buzzer timer
	accini();			//Initialize ACC MF TMP Sensor
	inituart(9600);		//Initialize UART 9600bps

/* MENU Loop */
    while(1){
	printf("UIEX Board Test");
	delay_ms(100);
	printf("\n\n%s\n", title[0]);
	printf("TEST No:%d\n", n);
    while(sw != 3){
    	sw = touch_sw();	    	//Read touch sensor
	    if(sw == NOTCH){
	    	on = 0;					//Finger off
	        delay_ms(50);			//Wait finger bound
	    }
    	if(sw == 0 && !on){ 		//SW1 ON
    		n++;					//Select test No.
    		if(n > 9) n = 1;		//Return No.1
       		printf("%s\n", title[n-1]);
    		printf("TEST No:%d\n", n);
     		//state = 1;
    		beep();					//Touch sound on
    		on = 1;					//Finger on
     	}
    }
    beep();							//Touch sound on
    delay_ms(50);					//Wait finger bound
    switch(n){						//Select execution test No.
    	case 1:
    		lcd_test();break;		//Go TEST1
    	case 2:
    		tch_sw();break;			//Go TEST2
    	case 3:
    		tch_slider();break;		//Go TEST3
    	case 4:
    		rtc_test();break;		//Go TEST4
    	case 5:
    		h_ind(); break;			//Go TEST5
    	case 6:
    		mf_ind();break;			//Go TEST6
    	case 7:
    		t_ind();break;			//Go TEST7
    	case 8:
    		sd_test();break;		//Go TEST8
    	case 9:
    		p_ind();break;			//Go Magnetic Field print out
     }
  }
  return 0;
}

/* SD ACCESS Test*/
void sd_test(void)
{
	signed char n, on = 0, sw = 0;

	puts("SD Music Player \n");
	puts("                \n");
	read_dir();						//Directory Read
	n = 1;							//File number count
 	while(1){
		sw = touch_sw();			//Read touch sensor
	    if(sw == NOTCH){
	   		on = 0;					//Finger off
			delay_ms(50);			//Wait finger bound
	   	}
	    else{
	    	if(on == 0){
	 			switch(sw){
					case 0:
						n--;break;	//File number count down
					case 1:
						n++;break;	//File number count up
					case 2:
						break;		//No operation
					case 3:
						beep();		//touch sound on
						play_m(n);	//Play start
				}
	 			on = 1;				//Finger on
	 			beep();				//touch sound on
	 			if(n > dir_bf[0][0]) n = 1;	//When file number count over
	 			if(n <= 0) n = dir_bf[0][0];//When file number count under
	    		puts("               \n");
	    		puts("               \n");
	 			printf("%s\n", dir_bf[n]);		//print file name
	 			printf("%9.3fKB\n", (float)dir_size[n]/1000);//print file size
	    	}
	    }
		delay_ms(50);
    }
}

/* Play music */
void play_m(signed char n)
{
	FRESULT rc;				//API access result
	WORD chn;				//remain file size
	int f_size;				//file size

	pwm_ini();				//PWM initialize
	open_file(dir_bf[n]);	//open file for play
	rc = pf_lseek(HEADER);	//Skip WAV file header
	rb_in = 0;				//initialize ring buffer control
	rb_out = 0;				//initialize ring buffer control
	//m_count = 0;			//initialize ring buffer control
	f_size = dir_size[n];
	do{
		if((rb_in - rb_out) < 3){		//Check ring buffer full
			rc = pf_read((void *)lbuf[rb_in % 3], 128, &chn); //Read file data
			if(rc) {put_rc(rc);break;}
			rb_in++;					//update in counter
			f_size -= 128;				//remain file size
			if((rb_out % 250) == 0){	//Display remain file siae
				printf("\n%9.3fKB\n", (float)f_size/1000);
			}
			if(touch_sw() == 2) break;	//STOP
		}
	}while(chn == 128);		//Check file end
	timer_ini();
}

/* open file */
void open_file(char *fname)
{
	FRESULT rc;
	char path[26];

	strcpy(path, "Music/");		//Music file directory
	strcat(path,fname);			//file path
	rc = pf_open(path);			//open file
	if(rc) put_rc(rc);
}

/* Read directory */
void read_dir(void)
{
	uint8_t res, n;
	FRESULT rc;

	res = disk_initialize();			//Initialize SD Card
	if(res) printf("res=%d\n", res);	//Err Output
	rc = pf_mount(&fs);					//Mount SD Card
	if(rc) put_rc(rc);
	rc = pf_opendir(&dir, "Music");		//open Music directory
	if(rc) put_rc(rc);
	for(n=1;;n++){
		rc = pf_readdir(&dir, &fno);	//read 1 file info
		if(rc) put_rc(rc);
		if(!fno.fname[0]) break;		//check end of file info
		strcpy(dir_bf[n],fno.fname);	//dir_bf[n] <- file name
		dir_size[n] = fno.fsize;		//dir_size[n] <- file size
	}
	dir_bf[0][0] = n - 1;				//dir_bf[0][0] <- number of file info
}


/* Put Response Message */
static void put_rc (FRESULT rc)
{
    static const char *message[] =
    {
        "OK\0", "DISK_ERR\0", "NOT_READY\0", "NO_FILE\0", "NO_PATH\0",
        "NOT_OPENED\0", "NOT_ENABLED\0", "NO_FILE_SYSTEM\0"
    };
    printf("%s\n\r", message[rc]);
}

/* LCD TEST */
void lcd_test(void){

	unsigned char ch = 0;

	write_lcd(0, 1);			//Clear LCD Display
	delay_ms(1);				//Wait execution

	cgwrite_lcd();
	write_lcd(0, 0x80);			//write 1'st line
	while(1){
		write_lcd(1, ch);		//Display character
		ch++;
		if((ch % 32) == 0){
			write_lcd(0, 0x80);	//write 1'st line
		}
		else{
			if((ch % 16) == 0) write_lcd(0, 0xc0); //write 2'nd line
		}
		delay_ms(200);			//Display delay
	}
}


/* Touch SENSOR TEST */
void tch_sw(void)
{
	signed char n = 1, sw, on = 0, toggle = 0;

    printf("\fTouch Switch   \n");
    printf("              ");
    n = 0;
    while(1){
	    sw = touch_sw();			//Read touch sensor
	    if(sw == NOTCH){				//Finger off
	    	on = 0;
	    	delay_ms(50);			//Wait finger bound
	    }
		for (n = 0; n < 4; n++){
			write_lcd(0, 0xc0 + n * 4);
	    	if((n == sw) && !on){	//Finger on
	    		toggle ^= 1 << sw;	//toggle LCD character
	    		LPC_GPIO0->DATA |= 0x80; //LED2 ON
	    		beep();				//Touch sound
	    		on = 1;
	    	}
	    	else{
	    		LPC_GPIO0->DATA &= ~0x80; //LED2 OFF
	    	}
	    	if((toggle & 1 << n) != 0){
	    		write_lcd(1, 0x78);			//Display X on LCD
	    	}
	    	else{
	    		write_lcd(1, 0xdb);			//Display O on LCD
	    	}
		}
		delay_ms(50);			//Wait finger bound
    }
}

void tch_slider(void)
{
	short int n, work;

	cgwrite_lcd();
	printf("\fTouch Slide %3d%%", 0);
	printf("                \n");
	while(touch_sw() < 0);	//Wait touch
	delay_ms(100);			//wait slider buf update
	slider_pos = 0;			//Zero clear slider position
	while(1){
		if(touch_sw() >= 0){
			printf("\fTouch Slide %3d%%", slider_pos);	//Display Finger Position
			work = slider_pos * 8;
			write_lcd(0, 0xc0);
			for(n = 0; n < work / 50; n++){
				write_lcd(1, 6);
			}
			write_lcd(1, work % 5);
			for(n = 0; n < 16 - work / 50; n++){
				write_lcd(1, 0);
			}
		}
		delay_ms(10);
	}
}

/* RTC Test */
void rtc_test(void)
{
	RTC rtc;

    init_rtc();				//initialize RTC IC
    GPIO_int_set();			//Setup 1 second interrupt

	printf("\f                                \f");
    while(1)
	{
    	if(p_f == 1){					//Check 1 second interrupt
    		rtc = get_rtc();			//get time from RTC IC
    		printf("%4d/%2d/%2d %s\n",rtc.year, rtc.month, rtc.mday, week + rtc.wday * 4);
    		printf("%4d:%2d:%2d\n",rtc.hour, rtc.min, rtc.sec);
    		LPC_GPIO0->DATA |= 0x80;	//LED2 on
     		p_f = 0;					//Clear 1 second flag
    	}
    	if(touch_sw() >= 0) rtc_set();	//Go time setup process
    	delay_ms(100);					//flag check delay
   		LPC_GPIO0->DATA &= ~0x80;		//LED2 off
	}
	return;
}

void rtc_set(void)
{
	RTC rtc;
	signed char n = 0, sw = 0, on = 0;
	short int dbuf[6];
	char cbuf[6] = {1,3,6,9,0x43,0x46};		//calendar_time column position

	rtc = get_rtc();						//get time RTC IC
    dbuf[1] = rtc.year - 2000;				//Move to work
    dbuf[2] = rtc.month;
    dbuf[3] = rtc.mday;
    dbuf[4] = rtc.hour;
    dbuf[5] = rtc.min;
    write_lcd(0, 0x0e);			//Display cursor
 	while((n >= 0) && (n < 6)){
		sw = touch_sw();		//Read touch sensor
	    if(sw == NOTCH){
	   		on = 0;				//Finger off
			delay_ms(50);		//Wait finger bound
	   	}
	    else{
			if(on == 0){		//Finger on
				switch(sw){
					case 0:
						n--;on = 1;break;	//Move column left
					case 1:
						n++;on = 1;break;	//Move column right
					case 2:
						dbufdn(dbuf, n);on = 1;break;	//increase value
					case 3:
						dbufup(dbuf, n);on = 1;break;	//decrease value
				}
				beep();									//touch sound on
				rtc.year = 2000 + dbuf[1];				//restore work
				rtc.month = dbuf[2];
				rtc.mday = dbuf[3];
				rtc.hour = dbuf[4];
				rtc.min = dbuf[5];
				rtc.sec = 00;
				/* ツェラーの公式　曜日の算出*/
				if(dbuf[2] < 3){
					dbuf[1]--;
					dbuf[2]+=12;
				}
				rtc.wday = (dbuf[1] + dbuf[1]/4 - dbuf[1]/100 + dbuf[1]/400 + (13 * dbuf[2] + 8)/5 + dbuf[3])%7;
				dbuf[1] = rtc.year - 2000;
				dbuf[2] = rtc.month;
				printf("%4d/%2d/%2d %s\n",rtc.year, rtc.month, rtc.mday, week + rtc.wday * 4);
				printf("%4d:%2d:%2d\n",rtc.hour, rtc.min, rtc.sec);
				write_lcd(0, 0x80 | cbuf[n]);	//Move cursor
			}
	    }
		delay_ms(50);			//Wait finger bound
    }
	put_rtc(rtc);
 	write_lcd(0, 0x0c);
}

/* Increase RTC calendar_time value */
void dbufup(short int *dbuf, char n)
{
	switch(n){
		case 1:		//year
			(*(dbuf+n))++;
			if(*(dbuf+n) > 99) *(dbuf+n) = 0;
			break;
		case 2:		//month
			(*(dbuf+n))++;
			if(*(dbuf+n) > 12) *(dbuf+n) = 1;
			break;
		case 3:		//day
			(*(dbuf+n))++;
			if(*(dbuf+n) > 31) *(dbuf+n) = 1;
			break;
		case 4:		//hour
			(*(dbuf+n))++;
			if(*(dbuf+n) > 23) *(dbuf+n) = 0;
			break;
		case 5:		//minute
			(*(dbuf+n))++;
			if(*(dbuf+n) > 59) *(dbuf+n) = 0;
			break;
	}
}

/* Decrease RTC calendar_time value */
void dbufdn(short int *dbuf, char n)
{
	switch(n){
		case 1:		//year
			(*(dbuf+n))--;
			if(*(dbuf+n) < 0) *(dbuf+n) = 99;
			break;
		case 2:		//month
			(*(dbuf+n))--;
			if(*(dbuf+n) < 1) *(dbuf+n) = 12;
			break;
		case 3:		//day
			(*(dbuf+n))--;
			if(*(dbuf+n) < 1) *(dbuf+n) = 31;
			break;
		case 4:		//hour
			(*(dbuf+n))--;
			if(*(dbuf+n) < 0) *(dbuf+n) = 23;
			break;
		case 5:		//minute
			(*(dbuf+n))--;
			if(*(dbuf+n) < 0) *(dbuf+n) = 59;
			break;
	}
}

/* ACC sensor test */
void h_ind(void)
{
	volatile uint8_t *p;
	int  x, y, z;
	float xangle, yangle;
	char count;

	printf("\fACC TEST       \n");
	printf("              \n");
	while(1){
		x = 0; y = 0; z = 0;
		for(count = 0; count < 8; count++){			//8回測定の平均を算出
			p = read_acc(ACCSAD, 0xa8, 6);			//加速度センサのリード
			x +=((short int)(*(p+1) * 256) + *p);
			y +=((short int)(*(p+3) * 256) + *(p+2));
			z +=((short int)(*(p+5) * 256) + *(p+4));
			delay_ms(1);		//平均するサンプリング間隔を1msにした
		}
		x /= 128;		//下位4ビットを切り捨て
		y /= 128;
		z /= 128;
		xangle = (float)atan((float)x / z) * (180 / PAI);		//x方向の傾きの算出
		yangle = (float)atan((float)y / z) * (180 / PAI);		//y方向の傾きの算出
		if((x >= 0) && (z < 0)) xangle = 180 + xangle;			//角度表示の変換
		if((x < 0) && (z < 0)) xangle = -180 + xangle;
		if((y >= 0) && (z < 0)) yangle = 180 +yangle;
		if((y < 0) && (z < 0)) yangle = -180 + yangle;
		printf("\nX:%+4.0f%c Y:%+4.0f%c\n",xangle,0xdf,yangle,0xdf);
		delay_ms(100);								//display delay
	}
}

/* Magnetic field sensor test */
void mf_ind(void)
{
	volatile uint8_t *p;
	int  x, y, z;
	float xyangle, xzangle;
	float mx, my, mz, mxy;
	char count;

	printf("\fMF TEST        \n");
	printf("              \n");
	while(1){
		x = 0; y = 0; z = 0;
		for(count = 0; count < 8; count++){
			p = read_acc(MFSAD, 0x83, 6);				//磁気センサのリード
			x +=((short int)(*p * 256) + *(p+1));		//8回平均の算出
			z +=((short int)(*(p+2) * 256) + *(p+3));
			y +=((short int)(*(p+4) * 256) + *(p+5));
			delay_ms(1);	//平均するサンプリング間隔1msにした
		}
		x /= 8;
		y /= 8;
		z /= 8;
		mx = (float)(x - MFXMIN) / (MFXMAX - MFXMIN) * 2 - 1.0;	//オフセットを加味してxを算出
		my = (float)(y - MFYMIN) / (MFYMAX - MFYMIN) * 2 - 1.0;	//オフセットを加味してｙを算出
		mz = (float)(z - MFZMIN) / (MFZMAX - MFZMIN) * 2 - 1.0;	//オフセットを加味してｚを算出
		mxy = sqrt(mx*mx + my*my);
		xyangle = (float)atan((float)mx / my) * (180 / PAI);	//方位の計算
		xzangle = (float)atan((float)mz / mxy) * (180 / PAI);	//伏角の計算
		xzangle *= -1;		//伏角の極性表示に合わせる
		if((mx < 0) && (my >= 0)) xyangle = 360 + xyangle;		//角度表示の変換
		if((mx >= 0) && (my < 0)) xyangle = 180 + xyangle;
		if((mx < 0) && (my < 0)) xyangle = 180 + xyangle;
		printf("\nA:%+4.0f%c D:%+4.0f%c\n",xyangle,0xdf,xzangle,0xdf);
		delay_ms(100);					//display delay
	}
}

/* Display Magnetic field sensor Value */
void p_ind(void)
{
	int mx = 0, my = 0, mz = 0;
	char count;
	volatile uint8_t *p;

	while(1){
		for(count = 0; count < 8; count++){
			p = read_acc(MFSAD, 0x83, 6);				//磁気センサのリード
			mx +=((short int)(*p * 256) + *(p+1));		//８回平均の算出
			mz +=((short int)(*(p+2) * 256) + *(p+3));
			my +=((short int)(*(p+4) * 256) + *(p+5));
			delay_ms(1);	//平均するサンプリング間隔1msにした
		}
		mx /= 8;
		my /= 8;
		mz /= 8;
		//printf("ax=%4d ay=%4d az=%4d\n",ax,ay,az);	//加速度表示
		printf("mx=%+4d my=%+4d mz=%+4d     \n",mx,my,mz);	//磁力表示
		delay_ms(100);		//Display delay
	}
}

/* Temperature sensor test */
void t_ind(void)
{
	volatile uint8_t *p;
	int  t;

	printf("\fTEMP TEST      \n");
	printf("              \n");
	while(1){
		p = read_acc(MFSAD, 0x31, 1);	//Read temperature
		t = *p + 25 ;		//下位4ビット切り捨て25を足す．．．STMのサンプルから持ってきた　何故か不明
		printf("\nTEMP:%d%cC\n",t, 0xdf);
		delay_ms(100);					//Display delay
	}
}

/* GPIO initialize */
void GPIOInit(void)
{
/* Initialize GPIO */
/*
 * PIO0_0:SW1(RESET)
 *
 * PIO1_6:RXD
 * PIO1_7:TXD
 *
 * PIO0_4:SCL
 * PIO0_5:SDA
 *
 * PIO1_0:touch SW1
 * PIO1_1:touch SW2
 * PIO1_2:touch SW3
 * PIO0_11:touch SW0
 * PIO1_8:RTC_INTA
 * PIO1_9:RTC_INTB
 * PIO1_4:SP1
 *
 * PIO1_5:MY ARM LED1
 * PIO0_7:MY ARM LED2
 * PIO0_1:MY ARM SW2
 * PIO0_3:SD_CD1		//Check SD CARD IN
 *
 * PIO0_9:SD_MOSI0(SPI)
 * PIO0_8:SD_MISO0(SPI)
 * PIO0_6:SD_SCK0(SPI)
 * PIO0_2:SD_CS
 *
 */
	LPC_GPIO0->DIR = 0b100010001100;	//Refer up
	LPC_GPIO1->DIR = 0b0000110111;		//Refer up
	LPC_GPIO0->DATA = 0x800;
	LPC_GPIO1->DATA = 0x17;

	LPC_IOCON->PIO0_2 = 0xd1;			//SSEL0を出力
	LPC_IOCON->PIO0_7 = 0xd0;
	LPC_IOCON->PIO1_8 = 0xd0;
	LPC_IOCON->R_PIO0_11 = 0xd1;
	LPC_IOCON->PIO1_9 = 0xd0;
	LPC_IOCON->R_PIO1_0 = 0xd1;
	LPC_IOCON->R_PIO1_1 = 0xd1;
	LPC_IOCON->R_PIO1_2 = 0xd1;
}

