﻿/*
===============================================================================
 Name        : pwm.c
 Author      : Ichiro Shirasaka
 Version     : Ver.1.0 2012.09.10
 Copyright   : Copyright (C) 2012 Ichiro Shirasaka
 Description : UIEXB TEST MAIN
===============================================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================
*/

#define FRQ 32000		//音楽再生サンプリング周波数

#include "LPC11xx.h"
#include "system_LPC11xx.h"

extern volatile uint8_t lbuf[4][128];	//Ring buffer
extern volatile int rb_in, rb_out;		//Ring buffer control

void pwm_ini(void)
{
	int cycle;		//PWM Cycle

	LPC_SYSCON->SYSAHBCLKCTRL |= (1 << 10);	//TMR32B1許可
	LPC_TMR32B1->MCR = (3 << 0);	//MAT0一致でタイマリセット、割り込み発生
	LPC_TMR32B1->EMR = (2 << 10);	//MAT3一致でMAT3出力を”１”
	LPC_IOCON->PIO1_4 = 0xd2;		//MAT3出力に設定
	LPC_TMR32B1->PR = 2;			//プリスケーラを1:3に設定
	cycle = 16000000 / FRQ - 1;		//周期32KHｚのタイマ値を計算
	LPC_TMR32B1->MR0 = cycle;		//周期を設定
	LPC_TMR32B1->PWMC = 0x8;		//MAT3をPWMモードに設定
	LPC_TMR32B1->TCR = 0x02;		//TCをリセット
	LPC_TMR32B1->TCR = 1;			//TCイネーブル
	NVIC_EnableIRQ(TIMER_32_1_IRQn);	//タイマ割り込み許可
}

void TIMER32_1_IRQHandler(void)
{
	uint8_t mdata;					//Music data
	static short int m_count = 0;	//Music data count

	if(((rb_in - rb_out) > 0)){
		mdata = (uint8_t)lbuf[rb_out % 3][m_count];	//read 1byte from ring buffer
		m_count++;
		LPC_TMR32B1->MR3 = (uint16_t)mdata*2 + 120;	//圧電サウンダの音が小さいので２倍拡大 音を聞きながらオフセットを調整
		if(m_count >= 128){		//Check block end of ring buffer
			m_count = 0;
			rb_out++;
		}
	}
	LPC_TMR32B1->IR = 0x01;		//MAT0割り込み要因をリセット
}



