﻿/*
===============================================================================
 Name        : rtc.c
 Author      : Ichiro Shirasaka
 Version     : Ver.1.0 2012.09.10
 Copyright   : Copyright (C) 2012 Ichiro Shirasaka
 Description : UIEXB TEST MAIN
===============================================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================
*/
#include "rtc.h"
#define SAD 0x64		//RTC slave address

char p_f = 0;			//1S interval flag for display

/* 時刻設定　初期化　*/
void init_rtc(void)
{
	uint8_t buf[11] = {0,0,0,0,1,1,12,0x23,0xa0,0};	//RTC IC initialize data
	RTC rtc;

    if(read_pon() == 1){
		rtc.year = 2012;
		rtc.month = 8;
		rtc.mday = 1;
		rtc.hour = 0;
		rtc.min = 0;
		rtc.sec = 0;
		rtc.wday = 3;
		put_rtc(rtc);					//initialize RTC(calendar_time data)
   		write_rtc_m(14, buf + 7, 2);	//initialize RTC
    }
}

/* 時刻の取得　*/
RTC get_rtc(void)
{
	volatile uint8_t *buf;
	RTC rtc;

	buf = read_rtc_m(0, 7);			//read RTC data
	rtc.sec = (buf[0] & 0x0F) + ((buf[0] >> 4) & 7) * 10; //store to rtc
	rtc.min = (buf[1] & 0x0F) + (buf[1] >> 4) * 10;
 	rtc.hour = (buf[2] & 0x0F) + ((buf[2] >> 4) & 3) * 10;
 	rtc.wday = (buf[3] & 0x07);
 	rtc.mday = (buf[4] & 0x0F) + ((buf[4] >> 4) & 3) * 10;
	rtc.month = (buf[5] & 0x0F) + ((buf[5] >> 4) & 1) * 10;
 	rtc.year =  2000 + (buf[6] & 0x0F) + (buf[6] >> 4) * 10;
  	return rtc;
}

/* 時刻の設定 */
void put_rtc(RTC rtc)
{
	uint8_t buf[7];

	buf[0] = rtc.sec / 10 * 16 + rtc.sec % 10;
	buf[1] = rtc.min / 10 * 16 + rtc.min % 10;
	buf[2] = rtc.hour / 10 * 16 + rtc.hour % 10;
	buf[3] = rtc.wday & 7;
	buf[4] = rtc.mday / 10 * 16 + rtc.mday % 10;
	buf[5] = rtc.month / 10 * 16 + rtc.month % 10;
	buf[6] = (rtc.year - 2000) / 10 * 16 + (rtc.year - 2000) % 10;
	write_rtc_m(0, buf, 7);			//write calendar_time to RTC
}

/* write rtc multi Byte */
void write_rtc_m(uint8_t subads, uint8_t *buf, uint8_t len)
{
	uint8_t count;

	for(count = 0; count < len; count++){	//multi byte write
		write_rtc(subads++, *buf++);
	}
}

/* write rtc 1Byte*/
void write_rtc(uint8_t subads, uint8_t data)
{
	/* Write SLA(W), address and one data byte */
	I2CWriteLength = 3;					//write 3byte
	I2CReadLength = 0;
	I2CMasterBuffer[0] = SAD;			//slave address
	I2CMasterBuffer[1] = (subads << 4);	/* sub address */
	I2CMasterBuffer[2] = data;			/* Data */
	I2CEngine();			//execute I2C engine
}

/* read rtc */
volatile uint8_t *read_rtc_m(uint8_t subads, uint8_t len)
{
	/* Write SLA(W), address, SLA(R), and read one byte back. */
	I2CWriteLength = 2;			//write 2byte
	I2CReadLength = len;		//read (len)byte
	I2CMasterBuffer[0] = SAD;	//slave address
	I2CMasterBuffer[1] = (subads << 4);		/* sub address */
	I2CMasterBuffer[2] = SAD | RD_BIT;		//set read bit
	I2CEngine();			//execute I2C engine
	return I2CSlaveBuffer;
}

/* read pon */
char read_pon(void)
{
	volatile uint8_t *pon_p;

	pon_p = read_rtc_m(15,1);		//read battery power down bit
	return (*pon_p >> 4) & 0x01;
}

/* Setup GPIO1(bit8) initerrupt */
void GPIO_int_set(void)
{
	LPC_GPIO1->IS &= ~(1 << 8);
	LPC_GPIO1->IBE &= ~(1 << 8);
	LPC_GPIO1->IEV |= (1 << 8);
	LPC_GPIO1->IE |= (1 << 8);
	NVIC_EnableIRQ(EINT1_IRQn);	//Enable interrupt
}

/* GPIO interrupt handler */
void PIOINT1_IRQHandler(void)
{
	LPC_GPIO1->IC |= (1 << 8);	//Reset interrupt cause
	p_f = 1;
}



