﻿/*
===============================================================================
 Name        : touch3.c
 Author      : Ichiro Shirasaka
 Version     : Ver.1.3 2012.09.10
 Copyright   : Copyright (C) 2012 Ichiro Shirasaka
 Description : UIEXB TEST MAIN
===============================================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================
*/
#include "gpio.h"
#include "touch.h"
#include <stdio.h>

#define TH 3		//定常ノイズとの閾値
#define THH 7		//タッチ判断の閾値
#define W1 -30		//スライダ処理　加重平均最小値
#define W2 20
#define W3 80
#define W4 130		//スライダ処理　加重平均最大値

void wait(void);
void cal_slider(void);

static long int adcini[4];	//非接触時ADC出力
static int touchval[4];		//タッチセンサのADC出力
static int init_done = 0;	//タッチセンサの初期化完了フラグ
static int nmax;			//THを越えていて４つのタッチセンサで一番大きな容量変化をしているパッド番号
int slider_pos = 0;			//タッチスライダ上の触れている位置の100分率


/* Tcouch swの判定インターフェース */
int touch_sw(void)
{
	return nmax;
}

/* Touch Padの初期化 */
void touch_ini(void)
{
	int count;
	int *p_t;

 		adcini[0] = 0;
 		adcini[1] = 0;
 		adcini[2] = 0;
 		adcini[3] = 0;
    	for(count = 0; count < 256; count++){
    		p_t = tsense();			//Read touch pad capacitance
        	adcini[0] += *p_t++;
        	adcini[1] += *p_t++;
        	adcini[2] += *p_t++;
        	adcini[3] += *p_t;
    		}
 		adcini[0] /= 256;			//calculate average after 256times read
 		adcini[1] /= 256;
 		adcini[2] /= 256;
 		adcini[3] /= 256;
    	init_done = 1;
  }

/* Touch switch検出 */
void touch_detect(void)
{
	int avg[4];
	int *p_t;
	int count;
	int swmax;
	int n, tch = 0;

	if(init_done == 0){		//initialize not done?
		nmax = 0;
		return;
	}
	avg[0] = 0;
	avg[1] = 0;
	avg[2] = 0;
	avg[3] = 0;
	for(count = 0; count < 16; count++)	{
		p_t = tsense();			//read touch pad
		avg[0] += *p_t++;
		avg[1] += *p_t++;
		avg[2] += *p_t++;
		avg[3] += *p_t;			//calculate average 16times read
	}
	for(n = 0; n < 4; n++){
		touchval[n] = (avg[n] / 16 - adcini[n]);
		if(touchval[n] < TH) touchval[n] = 0;	//TH以下はゼロ
	}
	tch = 0;
	for(n = 0; n < 4; n++){
		if(touchval[n] > THH){	//容量変化がTHH以上のとき触れたと判断
			tch = 1;
		}
	}
	if(tch == 0){
		nmax = NOTCH;				//SET NO Touch
		return;
	}
	swmax = 0;
	nmax = NOTCH;					//SET NO Touch
	for(n = 0; n < 4; n++){
		if(touchval[n] > swmax + TH){
			nmax = n;
			swmax = touchval[n];	//容量変化が最大のパッド位置を触れたSW番号と判断する
		}
	}
	cal_slider();					//スライダ処理を起動
	return;
}

/* Touch slider */
void cal_slider(void)
{
	static int buf[16];
	int n;
	int buf_avg;

	/* 加重平均処理 */
	slider_pos = (touchval[0] * W1 + touchval[1] * W2 + touchval[2] * W3 + touchval[3] * W4) / (touchval[0] + touchval[1] + touchval[2] + touchval[3]);
	if(slider_pos < 0) slider_pos = 0;			//0%以下は0%にする
	if(slider_pos > 100) slider_pos = 100;		//100%以上は100%にする
	for(n = 0, buf_avg = 0; n < 15; n++){		//移動平均処理
		buf[n] = buf[n + 1];
		buf_avg += buf[n];
	}
	buf[15] = slider_pos;
	buf_avg += buf[15];
	slider_pos = buf_avg / 16;
	return;
}

/* 静電容量計測処理 */
int *tsense(void)
{
	static int adc[4];

	LPC_IOCON->R_PIO0_11 = 0x42;	//AD0
	LPC_ADC->CR = 0x1000301;		//SELをAD0に設定
	while((LPC_ADC->DR[0] & (1<<31)) == 0);
	LPC_IOCON->R_PIO1_0 = 0x42;		//AD1
	LPC_ADC->CR = 0x1000302;		//SELをAD1に設定
	while((LPC_ADC->DR[1] & (1<<31)) == 0);
	LPC_IOCON->R_PIO1_1 = 0x42;		//AD2
	LPC_ADC->CR = 0x1000304;		//SELをAD2に設定
	while((LPC_ADC->DR[2] & (1<<31)) == 0);
	LPC_IOCON->R_PIO1_2 = 0x42;		//AD3
	LPC_ADC->CR = 0x1000308;		//SELをAD3に設定
	while((LPC_ADC->DR[3] & (1<<31)) == 0);
	LPC_ADC->CR &= 0xf8ffffff;
	wait();
	LPC_IOCON->R_PIO0_11 = 0x81;	//SW1 PADを充電
	LPC_IOCON->R_PIO1_0 = 0x81;		//SW2 PADを充電
	LPC_IOCON->R_PIO1_1 = 0x81;		//SW3 PADを充電
	LPC_IOCON->R_PIO1_2 = 0x81;		//SW4 PADを充電
	adc[0] = (LPC_ADC->DR[0] >> 6) & 0x3ff;
	adc[1] = (LPC_ADC->DR[1] >> 6) & 0x3ff;
	adc[2] = (LPC_ADC->DR[2] >> 6) & 0x3ff;
	adc[3] = (LPC_ADC->DR[3] >> 6) & 0x3ff;
	wait();
	return adc;
}

/* ADC initialize */
void adinit(void)
{
	LPC_SYSCON->PDRUNCFG &= ~(1<<4);	// ADコンバータに電力を供給
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<13);//ADコンバータにクロックを供給
	return;
}
/* Bit wait */
void wait(void)
{
	int count;
	for(count = 0; count < 100; count++);
}
