﻿/*
===============================================================================
 Name        : acc.c
 Author      : Ichiro Shirasaka
 Version     : Ver.1.0
 Copyright   : Copyright (C) 2012 Ichiro Shirasaka
 Description : XBee-Accelerator Main
===============================================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================
*/
#ifdef __USE_CMSIS
#include "LPC11xx.h"
#endif

#include <cr_section_macros.h>
#include <NXP/crp.h>
#include <stdio.h>
#include "uart.h"

// Variable to store CRP value in. Will be placed automatically
// by the linker when "Enable Code Read Protect" selected.
// See crp.h header for more information
__CRP const unsigned int CRP_WORD = CRP_NO_CRP ;

/********************************************************************
*
*  加速度センサの読み出し
*
*********************************************************************/
void GPIOInit(void);
void adinit(void);
int adconv(int port);
void delay_ms(int);

volatile int time_count = 0;
const char mes[6][80] = {
						"Xの値を最大にしてENTERキーを押してください\n",
						"Yの値を最大にしてENTERキーを押してください\n",
						"Zの値を最大してENTERキーを押してください\n",
						"Xの値を最小にしてENTERキーを押してください\n",
						"yの値を最小にしてENTERキーを押してください\n",
						"Zの値を最小にしてENTERキーを押してください\n"};

/* Main Loop */
int main(void)
{
	char input;
	short int i, n;
	short int max[3], min[3]; //X,Y,Z Max. Min.
	float data[3];

	GPIOInit();			//initialize GPIO
    SysTick_Config(SystemCoreClock / 1000);    //SysTick interrupt=1ms
    inituart(9600);		//initialize UART
    adinit();			//initilaize ADC

    while(1){
    	printf("\n加速度センサとXBeeのテストMenu\n");
    	printf("----------------------\n");
    	printf("加速度センサキャリブレーション:1\n");
    	printf("加速度センサデータ表示　　　:2\n");
    	printf("----------------------\n");
    	printf("No.?:");
    	input = getchar();

    	if(input == 0x31){				//Input = 1:calibration
    		for(n = 0; n < 3; n++){
    	   		printf(mes[n]);			//Print out message
    			while(!inkey()){		//Input Any Key
    				printf("X=%2.3d\r",adconv(n));	//順にX,Y,Zの値をリアルタイムに表示
        		}
    			max[n] = adconv(n);		//順にX,Y,ZのMAX値を格納
    			printf(mes[n + 3]);
    			while(!inkey()){
    					printf("X=%4d\r",adconv(n));	//順にX,Y,Zの値をリアルタイムに表示
    				}
    	   		min[n] = adconv(n);		//順にX,Y,ZのMIN値を格納
     		}
    	}
    	if(input == 0x32){				//Input = 2:Display data to Xbee
    	    while(!inkey()){			//abort if ANY key
    	    	for(i=0; i<3; i++){
    	    		/* データを-1.0～+1.0の範囲に正規化 */
    	    		data[i] = (float)(adconv(i) - min[i]) / (max[i] - min[i]) * 2 - 1.0;
    	    		printf("Port%d:%+1.3f ", i, data[i]); //Printout data
    	    	}
    	    	printf("\n");
    	    	delay_ms(10);
    		}
    	}
	}
	return 0;
}

/* AD converter */
int adconv(int port)
{
	unsigned int data = 0, work;
	char count;

	for(count = 0; count < 16; count++){	//16回の測定値を平均化
		while(((work = LPC_ADC->DR[port]) & (1<<31)) == 0);//Wait ADC end
		data += (work >> 6) & 0x3ff;
		delay_ms(1);
	}
	data /= 16;
	return data;
}

/* Delay ms */
void delay_ms(int time_ms)
{
    time_count = time_ms;
    while(time_count > 0);
}

__attribute__ ((section(".after_vectors")))
void SysTick_Handler(void)
{
   if(time_count > 0)
   {
	   time_count--;
   }
}

void adinit(void)
{
	/* PIOの機能を設定
	 * FUNC = 2:AD CONVERTERを選択
	 * MODE = 0:Pullup Pulldown無し
	 * HYS = 0:無効
	 * RESV = 1
	 * ADMODE = 0:Analogを有効
	 */
	LPC_IOCON->R_PIO0_11 = 0x42;		//PIO0_11をAD0入力に設定
	LPC_IOCON->R_PIO1_0 = 0x42;		//PIO1_0をAD1入力に設定
	LPC_IOCON->R_PIO1_1 = 0x42;		//PIO1_１をAD2入力に設定
	LPC_SYSCON->PDRUNCFG &= ~(1<<4);	// ADコンバータに電力を供給
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<13);	//ADコンバータにクロックを供給
	LPC_ADC->CR = 0x10207;			//AD0,1,2 CLKDIV=2 BURST=1
	return;
}

void GPIOInit(void)
{
	LPC_GPIO0->DIR = 0b100000000100;
	LPC_GPIO1->DIR = 0b0100000111;
	LPC_GPIO0->DATA = 0x0000;
	LPC_GPIO1->DATA = 0;

	LPC_IOCON->PIO0_7 = 0xd0;
	LPC_IOCON->PIO1_8 = 0xd0;
	LPC_IOCON->R_PIO0_11 = 0xd1;
	LPC_IOCON->PIO1_9 = 0xd0;
	LPC_IOCON->R_PIO1_0 = 0xd1;
	LPC_IOCON->R_PIO1_1 = 0xd1;
	LPC_IOCON->R_PIO1_2 = 0xd1;
}
