﻿/*
===============================================================================
 Name        : uart.c
 Author      : Ichiro Shirasaka
 Version     : Ver.1.0
 Copyright   : Copyright (C) 2012 Ichiro Shirasaka
 Description : XBee-Accelerator Main
===============================================================================
// ---- License Information -------------------------------
// Anyone can FREELY use this code fully or partially
// under conditions shown below.
// 1. You may use this code only for individual purpose,
//    and educational purpose.
//    Do not use this code for business even if partially.
// 2. You can copy, modify and distribute this code.
// 3. You should remain this header text in your codes
//   including Copyright credit and License Information.
// 4. Your codes should inherit this license information.
//=========================================================
// ---- Patent Notice -------------------------------------
// I have not cared whether this system (hw + sw) causes
// infringement on the patent, copyright, trademark,
// or trade secret rights of others. You have all
// responsibilities for determining if your designs
// and products infringe on the intellectual property
// rights of others, when you use technical information
// included in this system for your business.
//=========================================================
// ---- Disclaimers ---------------------------------------
// The function and reliability of this system are not
// guaranteed. They may cause any damages to loss of
// properties, data, money, profits, life, or business.
// By adopting this system even partially, you assume
// all responsibility for its use.
//=========================================================
*/
#ifdef __USE_CMSIS
#include "LPC11xx.h"
#endif

#include <cr_section_macros.h>
#include <NXP/crp.h>
#include <stdio.h>
#include "uart.h"

#define BUFMAX 80	//Buffer Max
#define CR 0xd		//Carriage return
#define LF 0xa		//Line feed

/* 低レベルIO出力関数 */
int __sys_write(int iFileHandle, char *pcBuffer, int iLength)
{
	while(iLength-- > 0)
	{
		send(*pcBuffer);					//1バイト送信
		if(*pcBuffer == LF)					//LF時CRも送る
		{
			send(CR);						//1バイト送信

		}
		pcBuffer++;
	}
	return 0;
}

/* 低レベルIO入力関数　*/
int __sys_readc(void)
{
	while(!(LPC_UART->LSR & 0x01));		// 受信待ち
	return LPC_UART->RBR; 				// 1バイト受信
}

/* Real time input */
char inkey(void)
{
	if((LPC_UART->LSR & 0x01) != 0){	//シリアルデータ受信
		return LPC_UART->RBR;
	}
	return 0;
}


/* シリアル送信 */
void send(unsigned char ch)
{
	while(!(LPC_UART->LSR & (1<<5)));	//送信バッファの空き待ち
	LPC_UART->THR = ch;					//送信バッファにライト
	return;
}


/* シリアル通信の設定 */
void inituart(uint32_t baurate)
{
	uint32_t dl;

	//IOポート機能の設定
	LPC_IOCON->PIO1_6 = 0x01;
	LPC_IOCON->PIO1_7 = 0x01;
	//UARTへのクロック供給
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<12);		//UARTを有効
	LPC_SYSCON->UARTCLKDIV = 1;				//UARTペリフェラルクロックの分周比を１に設定
	//ボーレートの設定
	dl = SystemCoreClock * LPC_SYSCON->SYSAHBCLKDIV /(16 * baurate * LPC_SYSCON->UARTCLKDIV);
	LPC_UART->LCR |= (1<<7);				//ディバイザラッチアクセスを許可
	LPC_UART->DLM = dl / 256;
	LPC_UART->DLL = dl % 256;
	LPC_UART->LCR &= ~(1<<7);				//ディバイザラッチアクセスを不許可
	//データフォーマットの設定（下記のデータフォーマットを設定します）
	/*WLS(1:0) = 0x03....データ長:8bit
	  SBS(2) = 0....STOPbit:1bit
	  PE(3)  = 0...Parity有効：無効
	  PS (5:4) = 0...Parity選択:odd Parity
	  BC(6)  = 0...Break制御:無効
	  DLAB(7) = 0...デバイザラッチアクセス:不許可*/
	LPC_UART->LCR = 0x03;					//データフォーマットの設定
	//バッファのクリア
	LPC_UART->FCR = 0x07;					//TXD,RXD FIFOクリア
	//LPC_UART->IER |= 0x01;				//RBR割り込み許可
    return;
}

